package code.lib

import code.model._
import net.liftweb.mapper._
import scala.collection.mutable.ListBuffer
import scala.language.existentials

object Statistics {

  def apply(questionnaire: Questionnaire) = {
    val stats = new Statistics(questionnaire)

    stats.fetch

    stats
  }

}

class Statistics(questionnaire: Questionnaire) {

  private val stats = ListBuffer[(Question, String, Int)]()
  
  def getData = stats.toList groupBy (_._1)

  def fetch {
    val questions = Question.findAll(By(Question.questionnaire, questionnaire))
    
    val data = questions map {
      question => {
        val answers = Answer.findAll(By(Answer.question, question))
        
        val answersByText = answers groupBy (_.text)
        (question, answersByText)
      }
    }
    
    data foreach {
      qta => {
        val question = qta._1
        
        qta._2 foreach {
          ta => {
            val text = ta._1
            val answers = ta._2.size
            
            question.answerType.get match {
              
              case "choice" => {
                val choice = Choice.find(By(Choice.id, text.get.toLong)).openOrThrowException(
                                    "it's not justified to use this, "
                                    +"but I don't know how to use map instead at this point"
                                  )
                stats += ((question, choice.text.get, answers))
              }
              
              case "multichoice" => {
                val parts = text.get.split(",")
                
                parts foreach {
                  part => {
                    val choice = Choice.find(By(Choice.id, part.toLong)).openOrThrowException(
                                    "it's not justified to use this, "
                                    +"but I don't know how to use map instead at this point"
                                  )
                    stats += ((question, choice.text.get, answers))
                  }
                }
                
              }
              case _ => {
                stats += ((question, text.get, answers))
              }
            }
          }
        }
      }
    }
  }

}
