package code.model

import net.liftweb.mapper._
import net.liftweb.common.Full
import scala.xml.Node

object Answer extends Answer with LongKeyedMetaMapper[Answer] {

  def fromXml(node: Node, questionnaire: Questionnaire, participation: Participation) = {
    val answer = Answer.create
    answer.participation(participation)

    val question = Question.find(By(Question.id, (node \ "@forQuestion").text.toLong),
      By(Question.questionnaire, questionnaire)).openOrThrowException(
                                    "it's not justified to use this, "
                                    +"but I don't know how to use map instead at this point"
                                  )
      
    answer.question(question)
    answer.text(node.text)

    answer
  }

}

class Answer extends LongKeyedMapper[Answer] with IdPK {
  def getSingleton = Answer

  object question extends MappedLongForeignKey(this, Question) {
    override def validSelectValues = Full(Question.findAll map (question => (question.id.get, question.text.get)))
  }
  
  object participation extends MappedLongForeignKey(this, Participation)

  object text extends MappedText(this)
}
